#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Wed Oct 26 10:28:53 2016

@author: albert
"""
import logging
import string
import sys
import re
try:
    from synapseqrcodes_version import __version__
    assert __version__
except ImportError:
    __version__ = "UNKNOWN"
try:
    from synapseqrcodes_version import __revision__
    assert __revision__
except ImportError:
    __revision__ = "UNKNOWN"

SPD = {
    '114-001629-000': {
        'Compliance Label': {
            'part number': '114-001618-000',
            'Description': 'Label containing model number for UL File'
            },
        'QR/Bar Code Label': {
            'part number': '114-001621-000',
            'Description': 'Label containing different addresses'
            }
        }
    }

PN_2_SPD = {
    'RF220SU': '114-002917-000',
    'RF220SU-EU': '114-002919-000',
    'SC020-010': '114-001629-000',
    'SC020-020': '114-001630-000',
    'SC020-030': '114-001656-000',
    'SC012-010': '114-002290-000',
    'SC012-020': '114-002292-000',
    'SC012-030': '114-002295-000',
    'SC012-040': '114-002298-000',
    'SL485-010': '114-002266-000',
    'SL485-020': '114-002286-000',
    'SL232-010': '114-002267-000',
    'SL232-020': '114-002287-000',
    'SN087-001': '114-002938-000',
    'EK2100-220': '114-002912-000',
    'EK5100-220': '114-002914-000',
    }

QRCODE_FORMATS = (
    '#01',
    '#02',
    '#03',
    '#04',
    '#05',
    '#06',
    '#07',
    '#08',
    'DOC-1000123',
    )
TYPE7_SALES_LEVEL_PART_NUMBERS = (
    'SN087-001',
    )
E12_SALES_LEVEL_PART_NUMBERS = (
    'SC012-010',
    'SC012-020',
    'SC012-030',
    'SC012-040',
    )
SNAPLINK_SALES_LEVEL_PART_NUMBERS = (
    'SL232-010',
    'SL232-020',
    'SL485-010',
    'SL485-020',
    )
E20_NO_MODEM_SALES_LEVEL_PART_NUMBERS = (
    'SC020-010',
    'SC020-030',
    'SS420-001',
    'LP-OEM-VS500',
    'SS420-002',
    'SS420-002E',
    'SS420-002EYE',
    'SS420-002V',
    'SS420-002LSI',
    'OC00G-001-00G',
    'OC00G-00G',
    )
E20_3G_MODEM_SALES_LEVEL_PART_NUMBERS = (
    'SC020-020',
    'SS450-001',
    'SS450-001E',
    'SS450-001EYE',
    'SS450-001V',
    )
E20_4G_MODEM_SALES_LEVEL_PART_NUMBERS = (
    'SC020-040',
    'SS450-002',
    'SS450-002E'
    )
E20_SALES_LEVEL_PART_NUMBERS = (
    E20_NO_MODEM_SALES_LEVEL_PART_NUMBERS +
    E20_3G_MODEM_SALES_LEVEL_PART_NUMBERS +
    E20_4G_MODEM_SALES_LEVEL_PART_NUMBERS
    )
RADIO_SALES_LEVEL_PART_NUMBERS = (
    'SM220UF1',
    'SM220UF1-DMX',
    'SM220UF1-XSBT',
    'RF200PF1',
    'RF200PD1',
    'RF220UF1',
    'RF220SU',
    'RF220SU-EU',
    )
EATON_SALES_LEVEL_PART_NUMBERS = (
    'DIM10-087-05',
    'SS420-003E',
    )
SALES_LEVEL_PART_NUMBERS = (
    RADIO_SALES_LEVEL_PART_NUMBERS +
    TYPE7_SALES_LEVEL_PART_NUMBERS +
    E12_SALES_LEVEL_PART_NUMBERS +
    E20_SALES_LEVEL_PART_NUMBERS +
    SNAPLINK_SALES_LEVEL_PART_NUMBERS +
    EATON_SALES_LEVEL_PART_NUMBERS
    )
LIGHTING_MODELS = (
    'DIM10-087-00',
    'DIM10-087-00-F',
    'DIM10-087-04',
    'DIM10-087-05',
    'DIM10-087-06',
    'DIM10-100-00',
    'DIM10-250-11',
    'DIM10-281-21',
    'DIM10-283-20',
    'AIM-121',
    'TL5-B1',
    'TL7-B1',
    'LP150-00',
    'OCF01-10T',
    'OCF01-1RT',
    'LP250V',
    'WSW-02',
    'WSW-08',
    'OCF00-1RN',
    )
E12_MODELS = (
    'E12',
    )
SNAP_LINK_MODELS = (
    'SNAPLINK',
    )
MODELS = (
    LIGHTING_MODELS +
    E12_MODELS +
    SNAP_LINK_MODELS
    )
PARAMETERS = (
    'qrcode format',
    'sales level part number',
    'date code',
    'ethernet mac address',
    'ethernet mac address last 6',
    'snap node address',
    'snap mac address',
    'wifi mac address',
    'model',
    'modem meid',
    'modem imei',
    'default password for user access',
    'default password for wifi access',
    'sim iccid',
    'batch size',
    'sequential code',
    'production order number',
    'HW version',
    )


def verify_date_code(date_code):
    if len(date_code) != 4:
        raise ValueError('date code must be 4 characters long: YYWW')
    try:
        year = int(date_code[:2])
    except ValueError:
        raise ValueError('YY in date code (YYWW) is not digits')
    if year < 7:
        raise ValueError(
            'YY in date code (YYWW) must not represent a year'
            ' 2007 or later, Synapse was founded in 2007'
            )
    try:
        week = int(date_code[2:])
    except ValueError:
        raise ValueError('WW in date code (YYWW) is not digits')
    if week > 52:
        raise ValueError('WW in date code (YYWW) is too large')


def verify_snap_node_address(snap_node_address):
    if len(snap_node_address) != 6:
        raise ValueError('snap node address length is not 6')
    if not all(ch in string.hexdigits for ch in snap_node_address):
        raise ValueError('snap node address contains not only hex digits')


def verify_ethernet_mac_address(ethernet_mac_address):
    if len(ethernet_mac_address) != 12:
        raise ValueError('ethernet MAC address length is not 12')
    if not all(ch in string.hexdigits for ch in ethernet_mac_address):
        raise ValueError('ethernet MAC address contains not hex digits')
    if ethernet_mac_address[:6] != '001C2C':
        raise ValueError('ethernet MAC address must start with 001C2C')


def verify_ethernet_mac_address_last_6_digits(ethernet_mac_address):
    if len(ethernet_mac_address) != 6:
        raise ValueError(
            'ethernet MAC address last 6 is not 6 characters long'
            )
    if not all(ch in string.hexdigits for ch in ethernet_mac_address):
        raise ValueError('ethernet MAC address contains not hex digits')


def verify_wifi_mac_address(wifi_mac_address):
    if len(wifi_mac_address) != 12:
        raise ValueError('wifi MAC address length is not 12')
    if not all(ch in string.hexdigits for ch in wifi_mac_address):
        raise ValueError('wifi MAC address contains not only hex digits')


def verify_snap_mac_address(snap_mac_address):
    if len(snap_mac_address) != 16:
        raise ValueError('snap MAC address length is not 16')
    if not all(ch in string.hexdigits for ch in snap_mac_address):
        raise ValueError('snap MAC address contains not only hex digits')


def verify_modem_meid(modem_meid):
    if len(modem_meid) != 14:
        raise ValueError('modem MEID length is not 14')
    if not all(ch in string.hexdigits for ch in modem_meid):
        raise ValueError('modem MEID contains not only hex digits')


def verify_modem_imei(modem_imei):
    if len(modem_imei) != 15:
        raise ValueError('modem IMEI length is not 15')
    if not all(ch in string.digits for ch in modem_imei):
        raise ValueError('modem IMEI contains not only digits')


def verify_sim_iccid(sim_iccid):
    if len(sim_iccid) != 20:
        raise ValueError('sim ICCID length is not 14')
    if not all(ch in string.digits for ch in sim_iccid):
        raise ValueError('sim ICCID contains not only digits')


def verify_batch_size(batch_size):
    if len(batch_size) > 4:
        raise ValueError('batch size length is more than 4')
    if not all(ch in string.digits for ch in batch_size):
        raise ValueError('batch size contains not only digits')


def verify_sequential_code(sequential_code):
    if len(sequential_code) > 4:
        raise ValueError('sequential code length is more than 4')
    if not all(ch in string.digits for ch in sequential_code):
        raise ValueError('sequential code contains not only digits')


def verify_production_order_number(production_order_number):
    if len(production_order_number) > 10:
        raise ValueError('production order number length is more than 10')


def verify_model(model):
    if not model in MODELS:
        raise ValueError('unrecognized model: %s' % repr(model))


def verify_sales_level_part_number(sales_level_part_number):
    if not sales_level_part_number in SALES_LEVEL_PART_NUMBERS:
        raise ValueError(
            'unrecognized sales level part number: %s' % repr(
                sales_level_part_number
                )
            )


def verify_qrcode_format(qrcode_format):
    if not qrcode_format in QRCODE_FORMATS:
        raise ValueError('unrecognized model: %s' % repr(qrcode_format))


def verify_parameter(parameter, value):
    """raises ValueError if parameter does not comply"""
    if not parameter in PARAMETERS:
        raise ValueError('unrecognized parameter: %s' % parameter)
    if parameter == 'date code':
        verify_date_code(value)
    elif parameter == 'ethernet mac address':
        verify_ethernet_mac_address(value)
    elif parameter == 'ethernet mac address last 6':
        verify_ethernet_mac_address_last_6_digits(value)
    elif parameter == 'wifi mac address':
        verify_wifi_mac_address(value)
    elif parameter == 'snap mac address':
        verify_snap_mac_address(value)
    elif parameter == 'snap node address':
        verify_snap_node_address(value)
    elif parameter == 'modem meid':
        verify_modem_meid(value)
    elif parameter == 'modem imei':
        verify_modem_imei(value)
    elif parameter == 'sim iccid':
        verify_sim_iccid(value)
    elif parameter == 'batch size':
        verify_batch_size(value)
    elif parameter == 'sequential code':
        verify_sequential_code(value)
    elif parameter == 'production order number':
        verify_production_order_number(value)
    elif parameter == 'model':
        verify_model(value)
    elif parameter == 'sales level part number':
        verify_sales_level_part_number(value)
    elif parameter == 'qrcode format':
        verify_qrcode_format(value)


def verify_output(output_dict):
    for key, value in output_dict.items():
        verify_parameter(key, value)
    return output_dict


def _synapse_qrcode_mac(scan):
    if scan.startswith('001C2C1F00'):
        return verify_output(
            {
                'snap mac address': scan,
                'snap node address': scan[-6:],
                'sales level part number': 'RF200PF1',
                })
    elif scan.startswith('001C2C1AD6'):
        return verify_output(
            {
                'snap mac address': scan,
                'snap node address': scan[-6:],
                'sales level part number': 'RF200PD1',
                })
    elif scan.startswith('001C2C1B26'):
        return verify_output(
            {
                'snap mac address': scan,
                'snap node address': scan[-6:],
                'sales level part number': 'RF220UF1',
                })
    elif scan.startswith('001C2C2AD6'):
        return verify_output(
            {
                'snap mac address': scan,
                'snap node address': scan[-6:],
                'sales level part number': 'RF220SU',
                })
    elif scan.startswith('001C2C2ED6'):
        return verify_output(
            {
                'snap mac address': scan,
                'snap node address': scan[-6:],
                'sales level part number': 'RF220SU-EU',
                })
    else:
        return verify_output(
            {
                'snap mac address': scan,
                'snap node address': scan[-6:],
                })


def _synapse_data_matrix_0202(scan):
    """
    SM220UF1
        14491001;260098C7;SM220UF1;0202
    SM220UF1-DMX
        14491001;260098FD;SM220UF1-DMX;0202
    SM220UF1-XSBT
        14491001;260098FD;SM220UF1-XSBT;0202
    """
    if not scan.endswith(";0202"):
        raise ValueError('not ending with ";0202"')
    scan_parts = scan.split(';')
    if len(scan_parts) == 4:
        return verify_output(
            {
                'date code': scan_parts[0][:4],
                'sequential code': scan_parts[0][4:],
                'snap mac address': '001C2C1B' + scan_parts[1],
                'snap node address': scan_parts[1][-6:],
                'sales level part number': scan_parts[2],
                })
    else:
        raise ValueError('wrong number of semicolons')


def _synapse_qrcode_1(scan):
    """
    #01:AAAAAA:BBBB:CCCCCCCCCCCC

    THE #01 IS RESERVED FOR LIGHTING PRODUCTS ONLY
    Note that this is one only of only two formats where there is no colon at
    the end of the data
    Field A = SNAP MAC ADDRESS (6 characters)
    Field B = SYNAPSE MANUFACTURING DATE CODE (4 characters,
              first 2 characters are year, second 2 characters are week:
              Example, 0451 means week 51 of year 2004)
    Field C = This is a variable length alphanumeric field that represents the
              model number
    Actual product example…
    #01:072982:1602:DIM10-250-11
    Some products that use this are:
    DIM10-250-11
    Many other lighting products
    """
    if not scan.startswith('#01:'):
        raise ValueError('not starting with "#01:"')
    if scan.endswith(':'):
        raise ValueError('colon at the end of the data')
    scan_parts = scan.split(':')
    lighting_model = scan_parts[3]
    if not lighting_model in LIGHTING_MODELS:
        raise ValueError(
            'unrecognized lighting model: %s' % repr(lighting_model)
            )
    if len(scan_parts) == 4:
        return verify_output(
            {
                'qrcode format': scan_parts[0],
                'snap node address': scan_parts[1],
                'date code': scan_parts[2],
                'model': lighting_model,
                })
    else:
        raise ValueError('wrong number of colons')


def _synapse_qrcode_2(scan):
    """
    #02:AAAAAAAAAAAAAAAAAAAA:BBBB:CCCCCCCCCCCC:\
        DDDDDDDDDDDDDDDD:EEEEEEEEEEEE:FFFFFFFFFFFFFF

    The #02 Format is for E20 products
    Note that this is one only of only two formats where there is no colon at
    the end of the data
    Field A = SALES LEVEL PART NUMBER (up to 20 characters)
    Field B = SYNAPSE MANUFACTURING DATE CODE (4 characters,
              first 2 characters are year, second 2 characters are week:
              Example, 0451 means week 51 of year 2004)
    Field C = ETHERNET MAC ADDRESS (12 characters)
    Field D = SNAP MAC ADDRESS (16 characters)
    Field E = WIFI MAC ADDRESS (12 characters)
    Field F = MODEM MEID (up to 14 characters); ‘0’ if not used
    Actual product example…
    #02:SC020‑020:1619:001C2C618B80:001C2C1B2608527E:00078006DE35:A1000042F1866B
    NOTE: due to a mistake initially, the E20’s without cell modem are
    formatted without field F at all.  QR code readers will have to be aware of
    the missing field.  Below is an example.
    All E20’s without cell modems will continue with this mistake for
    the sake of being consistent in format.
    #02:SC020-010:1618:001C2C62BB27:001C2C1B26085385:00078006E3DD
    Some products that use this are:
    E20 and variants
    """
    if not scan.startswith('#02:'):
        raise ValueError('not starting with "#02:"')
    if scan.endswith(':'):
        raise ValueError('colon at the end of the data')
    scan_parts = scan.split(':')
    if len(scan_parts) == 7:
        sales_level_part_number = scan_parts[1]
        # TBD is there a list of sales level part numbers of E20 variants?
        if len(sales_level_part_number) > 20:
            raise ValueError('sales level part number is too long')
        return verify_output(
            {
                'qrcode format': scan_parts[0],
                'sales level part number': sales_level_part_number,
                'date code': scan_parts[2],
                'ethernet mac address': scan_parts[3],
                'snap mac address': scan_parts[4],
                'wifi mac address': scan_parts[5],
                'modem meid': scan_parts[6],
                }
            )
    elif len(scan_parts) == 6:
        sales_level_part_number = scan_parts[1]
        # TBD is there a list of sales level part numbers of E20 variants?
        if len(sales_level_part_number) > 20:
            raise ValueError('sales level part number is too long')
        return verify_output(
            {
                'qrcode format': scan_parts[0],
                'sales level part number': sales_level_part_number,
                'date code': scan_parts[2],
                'ethernet mac address': scan_parts[3],
                'snap mac address': scan_parts[4],
                'wifi mac address': scan_parts[5],
                }
            )
    else:
        raise ValueError('wrong number of colons')


def _synapse_qrcode_3(scan):
    """
    #03:AAAAAAAAAAAAAAAAAAAA:BBBB:CCCCCCCCCCCC:\
        DDDDDDDDDDDDDDDD:0:FFFFFFFFFFFFFFFFFFFFFFFFFF:
    The #03 Format is for SNAP Link products
    Don’t forget the colon on the end of the data
    Field A = SALES LEVEL PART NUMBER (up to 20 characters)
    Field B = SYNAPSE MANUFACTURING DATE CODE (4 characters,
              first 2 characters are year, second 2 characters are week:
              Example, 0451 means week 51 of year 2004)
    Field C = ETHERNET MAC ADDRESS (12 characters); ‘0’ if not used
    Field D = SNAP MAC ADDRESS (16 characters)
    Field E = ‘0’
    Field F = SNAP Link MODEL (up to 26 characters)
    Example…
    #03:SN232-003:1610:0:001C2C1B26605BA5:0:SNAPLINK:
    Some products that use this are:
    SL232-003 (SNAP Link 232, FCC only)
    SL485-003 (SNAP Link 485, FCC only)
    SL232-004 (SNAP Link 232, CE)
    SL485-004 (SNAP Link 485, CE)
    """
    if not scan.startswith('#03:'):
        raise ValueError('not starting with "#03:"')
    if not scan.endswith(':'):
        raise ValueError('no colon at the end of the data')
    scan_parts = scan.split(':')
    if len(scan_parts) == 8:
        sales_level_part_number = scan_parts[1]
        # TBD is there a list of sales level part numbers
        # for SNAP Link products?
        if len(sales_level_part_number) > 20:
            raise ValueError('sales level part number is longer than 20')
        if scan_parts[5] != '0':
            raise ValueError('Field E is not zero')
        model = scan_parts[6]
        # TBD is there a list of models
        # for SNAP Link products?
        if len(model) > 26:
            raise ValueError('model is longer than 26')
        output = {
            'qrcode format': scan_parts[0],
            'sales level part number': sales_level_part_number,
            'date code': scan_parts[2],
            'snap mac address': scan_parts[4],
            'model': model,
            }
        if scan_parts[3] != '0':
            output['ethernet mac address'] = scan_parts[3]
        return verify_output(output)
    else:
        raise ValueError('wrong number of colons')


def _synapse_qrcode_4(scan):
    """
    #04:AAAAAAAAAAAAAAAAAAAA:BBBB:CCCCCCCCCCCC:\
        DDDDDDDDDDDDDDDD:0:FFFFFFFFFFFFFFFFFFFFFFFFFF:
    The #04 Format is for E12 products
    Don’t forget the colon on the end of the data
    Field A = SALES LEVEL PART NUMBER (up to 20 characters)
    Field B = SYNAPSE MANUFACTURING DATE CODE (4 characters,
              first 2 characters are year, second 2 characters are week:
              Example, 0451 means week 51 of year 2004)
    Field C = ETHERNET MAC ADDRESS (12 characters); ‘0’ if not used
    Field D = SNAP MAC ADDRESS (16 characters)
    Field E = ‘0’
    Field F = E12 MODEL (up to 26 characters)
    Example…
    #03:SC012-010:1610: 00078006DE35:001C2C1B26605BA5:0:E12:
    Some products that use this are:
    E12
    """
    if not scan.startswith('#04:'):
        raise ValueError('not starting with "#04:"')
    if not scan.endswith(':'):
        raise ValueError('no colon at the end of the data')
    scan_parts = scan.split(':')
    if len(scan_parts) == 8:
        sales_level_part_number = scan_parts[1]
        # TBD is there a list of sales level part numbers
        # for E12 products?
        if len(sales_level_part_number) > 20:
            raise ValueError('sales level part number is longer than 20')
        date_code = scan_parts[2]
        verify_date_code(date_code)
        ethernet_mac_address = scan_parts[3]
        verify_ethernet_mac_address(ethernet_mac_address)
        snap_mac_address = scan_parts[4]
        verify_snap_mac_address(snap_mac_address)
        field_e = scan_parts[5]
        if field_e != '0':
            raise ValueError('Field E is not zero')
        model = scan_parts[6]
        # TBD is there a list of models
        # for E12 products?
        if len(model) > 26:
            raise ValueError('model is longer than 26')
        return verify_output(
            {
                'qrcode format': scan_parts[0],
                'sales level part number': sales_level_part_number,
                'date code': date_code,
                'ethernet mac address': ethernet_mac_address,
                'snap mac address': snap_mac_address,
                'model': model,
                })
    else:
        raise ValueError('wrong number of colons')


def _synapse_qrcode_5(scan):
    """
    #05:AAAAAAAAAAAAAAAAAAAA:BBBBBBBBBBBBBBBBBBBB:
    The #05 Format is for SS420, SS450, and other SimplySNAP products
    Don’t forget the colon on the end of the data
    Field A = Default Password for User Access (12 - 20 characters)
    Field B = Default Password for WIFI Access (12 - 20 characters)
    Actual product example…
    #05:Sassy$Suzzyq78:Silent&Tim45:
    """
    if not scan.startswith('#05:'):
        raise ValueError('not starting with "#05:"')
    if not scan.endswith(':'):
        raise ValueError('no colon at the end of the data')
    scan_parts = scan.split(':')
    if len(scan_parts) == 4:
        return verify_output(
            {
                'qrcode format': scan_parts[0],
                'default password for user access': scan_parts[1],
                'default password for wifi access': scan_parts[2],
                })
    else:
        raise ValueError('wrong number of colons')


def _synapse_qrcode_6(scan):
    """
    #06:SC012-030:1650:63B704:
    The #06 Format is for E12 products w/o housing
    Don’t forget the colon on the end of the data
    Field A = SALES LEVEL PART NUMBER (up to 9 characters)
    Field B = SYNAPSE MANUFACTURING DATE CODE (4 characters,
              first 2 characters are year, second to characters are week:
              Example, 0451 means week 51 of year 2004)
    Field C = LAST 6 CHARACTERS OF THE ETHERNET MAC ADDRESS (6 characters)
    Example…
    #06:SC012-030:1650:63B704:
    Some products this is used on:
    SC012-030 (E12 SNAPconnect Gateway – w/o housing – North America)
    SC012-040 (E12 SNAPconnect Gateway – w/o housing – Europe)
    """
    if not scan.startswith('#06:'):
        raise ValueError('not starting with "#06:"')
    if not scan.endswith(':'):
        raise ValueError('no colon at the end of the data')
    scan_parts = scan.split(':')
    if not scan_parts[1] in E12_SALES_LEVEL_PART_NUMBERS:
        raise ValueError(
            'unrecognized type E12 sales level part number: %s' % repr(
                scan_parts[1]
                )
            )
    if len(scan_parts) == 5:
        return verify_output(
            {
                'qrcode format': scan_parts[0],
                'sales level part number': scan_parts[1],
                'date code': scan_parts[2],
                'ethernet mac address last 6': scan_parts[3],
                })
    else:
        raise ValueError('wrong number of colons')


def _synapse_qrcode_7(scan):
    """
    #07:SN087-001:1650:63B704:
    The #07 Format is for SN087 products
    Don’t forget the colon on the end of the data
    Field A = SALES LEVEL PART NUMBER (up to 9 characters)
    Field B = SYNAPSE MANUFACTURING DATE CODE (4 characters,
              first 2 characters are year, second to characters are week:
              Example, 0451 means week 51 of year 2004)
    Field C = LAST 6 CHARACTERS OF THE SNAP MAC ADDRESS (6 characters)
    Example…
    #07:SN087-001:1650:63B704:
    Some products this is used on:
    SN087-001
    """
    if not scan.startswith('#07:'):
        raise ValueError('not starting with "#07:"')
    if not scan.endswith(':'):
        raise ValueError('no colon at the end of the data')
    scan_parts = scan.split(':')
    if not scan_parts[1] in TYPE7_SALES_LEVEL_PART_NUMBERS:
        raise ValueError(
            'unrecognized type #07 sales level part number: %s' % repr(
                scan_parts[1]
                )
            )
    if len(scan_parts) == 5:
        return verify_output(
            {
                'qrcode format': scan_parts[0],
                'sales level part number': scan_parts[1],
                'date code': scan_parts[2],
                'snap node address': scan_parts[3],
                })
    else:
        raise ValueError('wrong number of colons')


def _synapse_qrcode_8(scan):
    """
    #08:AAAAAAAAAAAAAAAAAAAA:BBBB:CCCCCC:DDDDDD:EEEEEEEEEEEE:\
        FFFFFFFFFFFFFFFF:GGGGGGGGGGGGGGGGGGGG:
    The #08 Format is for E20 products with a 4G LTE modem
    Note: the Ethernet and SNAP MAC addresses were shortened
    to be able to keep the IMEI and ICCID in the QR code
    Don’t forget the colon on the end of the data
    Field A = SALES LEVEL PART NUMBER (up to 20 characters)
    Field B = SYNAPSE MANUFACTURING DATE CODE (4 characters,
              first 2 characters are year,
              second 2 characters are week
              Example: 0451 means week 51 of year 2004)
    Field C = ETHERNET MAC ADDRESS (6 characters)
    Field D = SNAP MAC ADDRESS (6 characters)
    Field E = WIFI MAC ADDRESS (12 characters)
    Field F = MODEM IMEI (up to 16 characters)
    Field G = SIM ICCID (up to 20 characters)
    Here is an example…
    #08:SC020-020:1619:618B80:08527E:00078006DE35:\
        356278079904498:89148000001081675984:
    Some products that use this are:
        E20
        SS450-002
    """
    if not scan.startswith('#08:'):
        raise ValueError('not starting with "#08:"')
    if not scan.endswith(':'):
        raise ValueError('no colon at the end of the data')
    scan_parts = scan.split(':')
    if len(scan_parts) == 9:
        if not scan_parts[1] in E20_4G_MODEM_SALES_LEVEL_PART_NUMBERS:
            raise ValueError(
                'unrecognized type E20 4G sales level part number: %s' % repr(
                    scan_parts[1]
                    )
                )
        return verify_output(
            {
                'qrcode format': scan_parts[0],
                'sales level part number': scan_parts[1],
                'date code': scan_parts[2],
                'ethernet mac address last 6': scan_parts[3],
                'snap node address': scan_parts[4],
                'wifi mac address': scan_parts[5],
                'modem imei': scan_parts[6],
                'sim iccid': scan_parts[7],
                }
            )
    else:
        raise ValueError('wrong number of colons')


def _synapse_qrcode_doc_1000123(scan):
    """full specification in the attached DOC-1000123"""
    if not re.match('\A..;.;10021361;', scan):
        raise ValueError('not a doc-1000123 QR code')
    scan_parts = scan.replace(';', ',').replace('of', ',').split(',')
    if len(scan_parts) == 15:
        return verify_output(
            {
                'qrcode format': 'DOC-1000123',
                'sales level part number': scan_parts[3],
                'production order number': scan_parts[5],
                'date code': scan_parts[8] + scan_parts[7],
                'ethernet mac address': scan_parts[11],
                'snap mac address': '001C2C1B26' + scan_parts[12],
                'snap node address': scan_parts[12],
                'sequential code': scan_parts[9],
                'batch size': scan_parts[10],
                'default password for user access': scan_parts[14],
                'default password for wifi access': scan_parts[13],
                }
            )
    elif len(scan_parts) == 13:
        return verify_output(
            {
                'qrcode format': 'DOC-1000123',
                'sales level part number': scan_parts[3],
                'production order number': scan_parts[5],
                'date code': scan_parts[8] + scan_parts[7],
                'HW version': scan_parts[12],
                'snap mac address': '001C2C1B26' + scan_parts[11],
                'snap node address': scan_parts[11],
                'sequential code': scan_parts[9],
                'batch size': scan_parts[10],
                }
            )
    else:
        raise ValueError('wrong number of parts: %d' % len(scan_parts))


def synapse_qrcode(scan):
    if scan.startswith('001C2C') and len(scan) == 16:
        return _synapse_qrcode_mac(scan)
    elif scan.endswith(";0202"):
        return _synapse_data_matrix_0202(scan)
    elif scan.startswith("#01:"):
        return _synapse_qrcode_1(scan)
    elif scan.startswith("#02:"):
        return _synapse_qrcode_2(scan)
    elif scan.startswith("#03:"):
        return _synapse_qrcode_3(scan)
    elif scan.startswith("#04:"):
        return _synapse_qrcode_4(scan)
    elif scan.startswith("#05:"):
        return _synapse_qrcode_5(scan)
    elif scan.startswith("#06:"):
        return _synapse_qrcode_6(scan)
    elif scan.startswith("#07:"):
        return _synapse_qrcode_7(scan)
    elif scan.startswith("#08:"):
        return _synapse_qrcode_8(scan)
    elif re.match('\A..;.;10021361;', scan):
        return _synapse_qrcode_doc_1000123(scan)
    else:
        raise ValueError('QRcode cannot be recognized')


def synapse_qrcode_loop():
    logging.basicConfig(level=logging.INFO)
    logging.info('synapseqrcodes version: %s', __version__)
    logging.info('synapseqrcodes revision: %s', __revision__)
    while True:
        try:
            scan = raw_input('\nScan Synapse QRcode (x to exit):')
            if scan.upper() == 'X':
                break
            qrcode = synapse_qrcode(scan)
            sys.stderr.write(repr(qrcode))
        except ValueError:
            logging.exception('ERROR')


if __name__ == "__main__":
    synapse_qrcode_loop()
